// Copyright (c) 2016, Shinichiro Abe. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

part of ducene;

abstract class Analyzer {
  Iterable<String> getTokens(String text);
}
class KeywordAnalyzer implements Analyzer {
  Iterable<String> getTokens(String text) => [text];
}
class WhitespaceAnalyzer implements Analyzer {
  Iterable<String> getTokens(String text) => text.split(' ').map((e) => e.toLowerCase());
}
class NGramAnalyzer implements Analyzer {
  final int _n;
  NGramAnalyzer(this._n);
  Iterable<String> getTokens(String text) {
    var ret = [];
    for (int i = 0; i < text.length; i++) {
      var endIndex = i + _n;
      if (!(endIndex > text.length)) ret.add(text.substring(i, endIndex).toLowerCase());
    }
    return ret;
  }
}
class EdgeNGramAnalyzer implements Analyzer {
  final int _n;
  EdgeNGramAnalyzer(this._n);
  Iterable<String> getTokens(String text) {
    var ret = [];
    for (int i = 0; i < _n; i++) {
      var endIndex = i + 1;
      if (!(endIndex > text.length)) ret.add(text.substring(0, endIndex).toLowerCase());
    }
    return ret;
  }
}
class ShingleAnalyzer implements Analyzer {
  final Analyzer _analyzer;
  final int _n;
  final String _separator;
  ShingleAnalyzer(this._analyzer, this._n, this._separator);
  Iterable<String> getTokens(String text) {
    var tokens = _analyzer.getTokens(text).toList();
    var ret = [];
    for (int i = 0; i < tokens.length; i++) {
      ret.add(_doShingle(tokens, i, _n).join(_separator));
    }
    return ret;
  }
  List<String> _doShingle(List<String> tokens, int i, int n) {
    var ret = [];
    for (int x = i; x < i + n; x++) {
      if (!(x > tokens.length - 1)) {
        ret.add(tokens[x]);
      }
    }
    return ret;
  }
}