// Copyright (c) 2016, Shinichiro Abe. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

part of ducene;

@deprecated
class Highlighter {
  static String getSnippet(BoolQuery query, String field,
      String text, Analyzer analyzer, {int size: 100, String separator: ''}) {
    List<String> ret = [];
    int endIndex = (text.length < size) ? text.length : size;
    Iterable<String> tokens = analyzer.getTokens(text.substring(0, endIndex));
    Iterable<TermQuery> qList = query.termQueries.where((e) => e.term.field == field);
    for (var token in tokens) {
      List<TermQuery> matched = qList.where((e) => e.term.text == token).toList();
      TermQuery tq = (matched.isEmpty) ? null : matched[0];
      if (tq != null) {
        ret.add('<b>');
        ret.add(token);
        ret.add('</b>');
      } else {
        ret.add(token);
      }
      ret.add(separator);
    }
    if (separator.length > 0) ret.removeLast();
    return ret.join();
  }
  static String getHighlightSnippet(BoolQuery query, String field, String text) {
    List<String> ret = [];
    List<String> tokens = _newUniGramAnalyzer().getTokens(text);
    List<String> hl_tokens = new HighlightUniGramAnalyzer().getTokens(text);
    assert(tokens.length == hl_tokens.length);
    Iterable<TermQuery> qList = query.termQueries.where((e) => e.term.field == field);
    int i = 0;
    for (var token in tokens) {
      List<TermQuery> matched = qList.where((e) => e.term.text == token).toList();
      TermQuery tq = (matched.isEmpty) ? null : matched[0];
      if (tq != null) {
        ret.add('<b>');
        ret.add(hl_tokens[i]);
        ret.add('</b>');
      } else {
        ret.add(hl_tokens[i]);
      }
      i++;
    }
    return ret.join();
  }
  static NGramAnalyzer _newUniGramAnalyzer() {
    return new NGramAnalyzer(1);
  }
}
class HighlightUniGramAnalyzer implements Analyzer {
  Iterable<String> getTokens(String text) {
    var ret = [];
    for (int i = 0; i < text.length; i++) {
      var endIndex = i + 1;
      if (!(endIndex > text.length)) ret.add(text.substring(i, endIndex)); // not lowerCased
    }
    return ret;
  }
}
class FieldHighlight {
  static String getSnippet(BoolQuery query, String field, String text) {
    var terms = getTerms(query, field);
    var offsets = getOffsets(terms, text);
    return markTags(offsets, text, '<b>', '</b>');
  }
  static List<String> getTerms(BoolQuery query, String field) {
    return query.termQueries
        .where((e) => e.term.field == field)
        .map((e) => e.term.text)
        .toList();
  }
  static String markTags(List<Map<int,int>> offset, String text, String preTag, String postTag) {
    List<int> startOffsets = offset.map((e) => e.keys.toList()[0]).toList();
    List<int> endOffsets = offset.map((e) => e.values.toList()[0]).toList();
    StringBuffer sb = new StringBuffer();
    int i = 0;
    for (var token in new HighlightUniGramAnalyzer().getTokens(text)) {
      if (startOffsets.contains(i)) {
        sb.write(preTag);
      } else if (endOffsets.contains(i)) {
        sb.write(postTag);
      }
      sb.write(token);
      i++;
    }
    if (endOffsets.contains(i)) {
      sb.write(postTag);
    }
    return sb.toString();
  }
  static List<Map<int,int>> getOffsets(List<String> terms, String text) {
    // dedupe terms
    Set<String> termsSet = new Set.from(terms);
    // tokenize both of param
    var termsList = [];
    for (var term in termsSet) {
      List<String> tokens = new NGramAnalyzer(1).getTokens(term);
      termsList.add(tokens);
    }
    List<String> textTokens = new NGramAnalyzer(1).getTokens(text);
    // try to match both of param
    var list = <Map<int,int>>[];
    for (List<String> tokens in termsList) {
      int startIndex = textTokens.join().indexOf(tokens.join());
      if (startIndex == -1) continue;
      var endIndex = startIndex + tokens.length;
      if (endIndex > textTokens.length) continue;
      var textSequence = textTokens.getRange(startIndex, endIndex);
      int i = 0;
      bool found = false;
      for (var c in textSequence) {
        if (c == tokens[i]) {
          found = true;
        } else {
          found = false;
          break;
        }
        i++;
      }
      if (found) {
        Map m = {};
        m[startIndex] = endIndex;
        list.add(m);
      }
    }
    // remove within/overlap to extract offsets
    var clone = <Map<int,int>>[];
    for (Map e in list) {
      clone.add(e);
    }
    var ret = <Map<int,int>>[];
    int curPos = 0;
    for (Map cur in list) {
      int curStart = cur.keys.toList()[0];
      int curEnd = cur.values.toList()[0];
      bool within = false;
      int pos = 0;
      for (Map x in clone) {
        int s = x.keys.toList()[0];
        int e = x.values.toList()[0];
        if (curPos != pos && s <= curStart && curEnd <= e) {
          within = true;
          break;
        }
        pos++;
      }
      bool overlap = false;
      pos = 0;
      for (Map x in clone) {
        int s = x.keys.toList()[0];
        int e = x.values.toList()[0];
        if (curPos != pos && (s <= curStart && curStart <= e)) {
          overlap = true;
          break;
        }
        pos++;
      }
      if (!within && !overlap) {
        ret.add(cur);
      }
      curPos++;
    }
    // sort
    ret.sort((a, b) => a.keys.toList()[0] - b.keys.toList()[0]);
    return ret;
  }
}