// Copyright (c) 2016, Shinichiro Abe. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

import 'package:ducene/ducene.dart';
import 'package:test/test.dart';

void main() {
  group('A group of stats', () {
    Analyzer kwAnalyzer = new KeywordAnalyzer();

    test('test correct stats', () {
      IndexDirectory indexDirectory = new RAMIndexDirectory();
      IndexWriter writer = new IndexWriter(indexDirectory);

      List<Document> d1 = [];
      for (int i = 0; i < 10; i++) {
        Document doc = new Document();
        doc.append('id', i.toString(), kwAnalyzer);
        doc.append('text', 'text', kwAnalyzer);
        doc.append('quantity', '3', kwAnalyzer);
        d1.add(doc);
      }//30
      List<Document> d2 = [];
      for (int i = 10; i < 15; i++) {
        Document doc = new Document();
        doc.append('id', i.toString(), kwAnalyzer);
        doc.append('text', 'text', kwAnalyzer);
        doc.append('quantity', '5', kwAnalyzer);
        d2.add(doc);
      }//25
      var docs = <Document>[];
      docs.addAll(d1);
      docs.addAll(d2);

      writer.write(docs);

      IndexReader reader = DirectoryReader.open(indexDirectory);
      IndexSearcher searcher = new IndexSearcher(reader);

      BoolQuery q = new BoolQuery(Op.or);
      q.append('text', 'text', kwAnalyzer);
      TopDocs hits = searcher.search(q, 100, new MatchScoreSort(q));

      expect(hits.totalHits, 15);

      var quantity_stats = FieldStats.getStats(hits.docSet,'quantity',reader);
      expect(quantity_stats['min'], 3);
      expect(quantity_stats['max'], 5);
      expect(quantity_stats['count'], 15);
      expect(quantity_stats['sum'], 55);
      expect(quantity_stats['mean'], 55/15);
    });

  });
}

